@@ -109,7 +109,8 @@ var MapView = GObject.registerClass({
         'going-to-user-location': {},
         'gone-to-user-location': {},
         'view-moved': {},
-        'marker-selected': { param_types: [Champlain.Marker] }
+        'marker-selected': { param_types: [Champlain.Marker] },
+        'map-type-changed': { param_types: [GObject.TYPE_STRING] }
     },
 }, class MapView extends GtkChamplain.Embed {
 
@@ -322,7 +323,7 @@ var MapView = GObject.registerClass({
         this._mapType = mapType;
 
         if (mapType !== MapType.LOCAL) {
-            if (mapType === MapType.AERIAL && Service.getService().tiles.aerial)
+            if (mapType === MapType.AERIAL)
                 this.view.map_source = MapSource.createAerialSource();
             else
                 this.view.map_source = MapSource.createStreetSource();
@@ -332,15 +333,13 @@ var MapView = GObject.registerClass({
                 this.view.add_child(this._attribution);
             }
 
-            this._attribution.setSource(this.view.map_source);
-
             Application.settings.set('map-type', mapType);
         } else {
             let renderer = new Champlain.ImageRenderer();
             let source = new Maps.FileTileSource({
                 path: Utils.getBufferText(Application.application.local_tile_path),
                 renderer: renderer,
-                tile_size: 512
+                tile_size: Application.application.local_tile_size || 512
             });
             try {
                 source.prepare();
@@ -358,6 +357,8 @@ var MapView = GObject.registerClass({
         }
 
         overlay_sources.forEach((source) => this.view.add_overlay_source(source, 255));
+
+        this.emit("map-type-changed", mapType);
     }
 
     toggleScale() {
@@ -489,19 +490,9 @@ var MapView = GObject.registerClass({
     }
 
     _storeLocation() {
-        let zoom = this.view.zoom_level;
+        Application.settings.set('zoom-level', this.view.zoom_level);
         let location = [this.view.latitude, this.view.longitude];
-
-        /* protect agains situations where the Champlain view was already
-         * disposed, in this case zoom will be set to the GObject property
-         * getter
-         */
-        if (!isNaN(zoom)) {
-            Application.settings.set('zoom-level', zoom);
-            Application.settings.set('last-viewed-location', location);
-        } else {
-            Utils.debug('Failed to extract location to store');
-        }
+        Application.settings.set('last-viewed-location', location);
     }
 
     _goToStoredLocation() {
